package internetBanking;

/**
 * DesignByContract is a superclass providing some utility methods
 * (i.e. require and ensure) for mimicing design by contract.
 * Subclasses should call require and ensure at appropriate times. 
 * 
 * Make sure that assertions are enables when compiling or running.
 * in Eclipse
 * 1. Preferences (Main menu ... Eclipse)
 * 2. Java
 * 3. Installed JREs
 * 4. Select your JRE/JDK
 * 5. Press Edit...
 * 6. Default VM arguments
 * 7. Add -ea
 */

/**
 * @author sergedemeyer - September 2015
 *
 */
public class DesignByContract {

	/**
	 * Call require to check a pre-condition.
	 * 
	 * @param errorMsg
	 *            -- pass intention revealing errorMessage
	 */
	public void require(boolean preConditionValue, String errorMsg) {
		assert preConditionValue : "Pre-condition violation: " + errorMsg;
	}

	/**
	 * Call ensure to check a post-condition.
	 * 
	 * @param errorMsg
	 *            -- pass intention revealing errorMessage
	 */
	public void ensure(boolean postConditionValue, String errorMsg) {
		assert postConditionValue : "Post-condition violation: " + errorMsg;
	}

	/**
	 * Call ensure to check a post-condition.
	 * 
	 * @param errorMsg
	 *            -- pass intention revealing errorMessage
	 */
	public void invariant(boolean invariantValue, String errorMsg) {
		assert invariantValue : "Invariant violation: " + errorMsg;
	}

	/**
	 * For testing purposes actually do a pre-condition violation
	 */
	public void triggerPreconditionViolation() {
		this.require(false, "Forced pre-condition.");
	}

	/**
	 * For testing purposes actually do a post-condition violation
	 */
	public void triggerPostconditionViolation() {
		this.ensure(false, "Forced post-condition.");
	}

	/**
	 * For testing purposes actually do a post-condition violation
	 */
	public void triggerInvariantViolation() {
		this.invariant(false, "Forced invariant.");
	}

}
