package internetBanking.tests;

/**
 * Internet Banking Simulation -- Unit Tests.
 * Simple Java project to illustrate the power of refactorings in modern IDEs
 */

/**
 * @author sergedemeyer - September 2015
 *
 */

import junit.framework.*;

import java.util.ArrayList;
import java.util.List;

import internetBanking.Bank;
import internetBanking.Customer;

public class InternetBankingTests extends TestCase {
	public Customer customer = new Customer(33);
	public Bank bank = new Bank();
	public int fromAccountNr = bank.createAccountForCustomer(customer);
	public int toAccountNr = bank.createAccountForCustomer(customer);

	public static Test suite() {
		TestSuite testSuite = new TestSuite();
		testSuite.addTestSuite(AccountTest.class);
		testSuite.addTestSuite(CustomerTest.class);
		testSuite.addTestSuite(DesignByContractTests.class);
		testSuite.addTestSuite(InternetBankingTests.class);
		return testSuite;
	}

	/**
	 * testSmallPop: call constructor + getters/setters + operations +
	 * destructor See Robert V. Binder. Testing Object-Oriented Systems.
	 * Addison-Wesley, 2000
	 */
	public void testSmallPop() {
		assertTrue(bank.valid_customer(customer));
		assertEquals(1, fromAccountNr);
		assertTrue(bank.customerMayAccess(customer, fromAccountNr));
		assertEquals(0, bank.seeBalance(customer, fromAccountNr));
		assertTrue(bank.customerMayAccess(customer, toAccountNr));
		assertEquals(0, bank.seeBalance(customer, toAccountNr));
	}

	public void testSingleTransfer() {
		bank.transfer(customer, fromAccountNr, toAccountNr, 100);
		assertEquals(-100, bank.seeBalance(customer, fromAccountNr));
		assertEquals(+100, bank.seeBalance(customer, toAccountNr));

		bank.transfer(customer, toAccountNr, fromAccountNr, 100);
		assertEquals(0, bank.seeBalance(customer, fromAccountNr));
		assertEquals(0, bank.seeBalance(customer, toAccountNr));
	}

	public void testMultipleTransfers() {
		for (int i = 1; i <= 10; i++) {
			bank.transfer(customer, fromAccountNr, toAccountNr, 100);
			assertEquals(-100 * i, bank.seeBalance(customer, fromAccountNr));
			assertEquals(+100 * i, bank.seeBalance(customer, toAccountNr));
		}

		bank.transfer(customer, toAccountNr, fromAccountNr, 1000);
		assertEquals(0, bank.seeBalance(customer, fromAccountNr));
		assertEquals(0, bank.seeBalance(customer, toAccountNr));
	}

	public void testMultipleTransfersConcurrent() {
		List<Thread> threads = new ArrayList<Thread>();

		for (int i = 1; i <= 10; i++) {
			Runnable task = new RunnableTransfer(this, i);
			Thread worker = new Thread(task);
			worker.setName("RunnableTransfer_Nr_" + String.valueOf(i));
			worker.start();
			threads.add(worker);
		}
		int running = 0;
		do {
			running = 0;
			for (Thread thread : threads) {
				if (thread.isAlive()) {
					running++;
				}
			}
			System.out.println("We have " + running + " running threads. ");
		} while (running > 0);

	}

	/*
	*/

}
