package internetBanking;

/**
 * @author sergedemeyer - September 2015 An account is a dataholder for a
 *         balance (the amount of money available) and is identified by means of
 *         an accountNr (a unique identifier)
 */

public class Account extends Lockable {
	private int _accountNr = 0, _balance = 0;
	private int _workingBalance = 0;

	public Account(int accountNr) {
		this._accountNr = accountNr;
	}

	/**
	 * @return the _accountNr
	 */
	public int get_accountNr() {
		return _accountNr;
	}

	/**
	 * @return the _balance, used as a peeking method for checking
	 *         pre-conditions within contracts
	 */
	protected synchronized int peek_balance() {
		return _balance;
	}

	/**
	 * @param transaction
	 *            identifies the transaction accessing the balance
	 * @return the _balance
	 */
	public synchronized int get_balance(int transaction) {
		this.require(this.isLockedBy(transaction), "Lock is required to operate on an account.");
		return _workingBalance;
	}

	/**
	 * @param balance
	 *            the amount to be added to _balance
	 * @param transaction
	 *            identifies the transaction accessing the balance
	 */
	public synchronized void inc_balance(int balance, int transaction) {
		this.require(this.isLockedBy(transaction), "Lock is required to operate on an account.");
		this._workingBalance = _workingBalance + balance;
	}

	/**
	 * 
	 */
	@Override
	protected void copyToWorkingState() {
		this._workingBalance = this._balance;
	}

	/**
	 * 
	 */
	@Override
	protected void commitWorkingState() {
		this._balance = this._workingBalance;
	}

	/**
	 * 
	 */
	@Override
	protected void clearWorkingState() {
		this._workingBalance = 0;
	}

}
